<?php
	require_once('sql.class.php');
	class Attachment implements JsonSerializable{
		public $smf;
		public $id;
		public $id_thumb;
		public $id_msg;
		public $id_member;
		public $attachment_type;
		public $filename;
		public $file_hash;
		public $fileext;
		public $size;
		public $downloads;
		public $width;
		public $height;
		public $mime_type;
		public $approved;
		public function __construct($id,$smf){
			global $db_prefix;
			$this->smf = $smf;
			$this->id = $id;
			$post = $smf->sql->query("
				SELECT	id_thumb,
						id_msg,
						id_member,
						attachment_type,
						filename,
						file_hash,
						fileext,
						size,
						downloads,
						width,
						height,
						mime_type,
						approved
				FROM {$db_prefix}attachments
				WHERE  id_attach = ?;
			",'i',$id)->assoc_result;
			if($post){
				foreach($post as $k => $v){
					$this->$k = $v;
				}
			}
		}
		public function jsonSerialize(){
			$a = array(
				'id'=>intval($this->id),
				'name'=>$this->filename,
				'href'=>$this->href
			);
			if($this->member){
				$a['member'] = array(
					'id'=>intval($this->member->id),
					'name'=>$this->member->name,
					'real_name'=>$this->member->real_name
				);
			}
			if($this->thumbnail){
				$a['thumbnail'] = $this->thumbnail;
			}
			if($this->post){
				$a['post'] = array(
					'id'=>intval($this->post->id),
					'subject'=>$this->post->subject
				);
				$a['topic'] = array(
					'id'=>intval($this->topic->id),
					'subject'=>$this->topic->subject
				);
			}
			return $a;
		}
		public function __toString(){
			return "[Attachment {$this->id}] {\n  Name: {$this->filename}\n  Size: {$this->size}\n  Downloads: {$this->downloads}\n}";
		}
		public function __get($name){
			global $db_prefix;
			switch($name){
				case 'id_topic':
					return $this->post?$this->post->id_topic:false;
				break;
				case 'name':
					return $this->filename;
				break;
				case 'thumbnail':
					return $this->id_thumb>0?$this->smf->attachment($this->id_thumb):false;
				break;
				case 'post':
					return $this->id_msg>0?$this->smf->post($this->id_msg):false;
				break;
				case 'topic':
					return $this->id_topic?$this->smf->topic($this->id_topic):false;
				break;
				case 'member':
					return $this->id_member>0?$this->smf->member($this->id_member):false;
				break;
				case 'path':
					$path = $this->filename;
					if(!empty($this->file_hash)){
						$path = $this->id.'_'.$this->file_hash;
					}
					$path = SMF_PATH.'/attachments/'.$path;
					return file_exists($path)?$path:false;
				break;
				case 'href':
					global $boardurl;
					return $this->id_topic?"{$boardurl}/?action=dlattach;topic=".$this->id_topic.";attach=".$this->id:'';
				break;
				case 'mime':
					$ret = false;
					if($this->path){
						$f = finfo_open(FILEINFO_MIME_TYPE);
						$ret = finfo_file($f, $this->path);
						finfo_close($f);
					}
					return $ret;
				break;
				case 'datauri':
					return $this->path?'data:'.$this->mime.';base64,'.base64_encode($this->raw):false;
				break;
				case 'raw':
					return $this->path?file_get_contents($this->path):false;
				break;
				case 'permissions':
					return $this->permissions();
				break;
				case 'visible':
					return $this->visible();
				break;
			}
		}
		public function read($member=null){
			$member = is_null($member)?$this->smf->current_user:$member;
			global $db_prefix;
			$this->smf->sql->query("
				INSERT INTO {$db_prefix}log_topics (
					id_msg,
					id_member,
					id_topic
				)
				VALUES (?,?,?)
				ON DUPLICATE KEY UPDATE
					id_msg = ?
			",'iiii',
				$this->id,
				$member->id,
				$this->topic->id,
				$this->id
			)->execute();
		}
		public function permission($permission,$member=null){
			return $this->post->permission($permission,$member);
		}
		public function permissions($member=null){
			return $this->post->permissions($member);
		}
		public function visible($member=null){
			return $this->post->visible($member);
		}
	}
?>