<?php
	require_once('sql.class.php');
	/**
	* Category class. Meant for interfacing with an SMF category.
	*
	* @module smf
	* @class Category
	* @constructor
	*/
	class Category implements JsonSerializable{
		/**
		* The ID of the category
		* 
		* @property id
		* @type {Integer}
		* @public
		* @required
		*/
		public $id;
		/**
		* The name of the category
		* 
		* @property name
		* @type {String}
		* @public
		* @required
		*/
		public $name;
		/**
		* The SMF object that is the parent of the category
		* 
		* @property smf
		* @type {SMF}
		* @private
		* @required
		*/
		public $smf;
		public function __construct($id,$smf){
			global $db_prefix;
			$this->id = $id;
			$this->smf = $smf;
			$category = $this->smf->sql->query("
				SELECT name
				FROM {$db_prefix}categories c
				WHERE id_cat = ?;
			",'i',$this->id)->assoc_result;
			if(!$category){
				$this->name = '[ROOT]';
			}else{
				foreach($category as $k => $v){
					$this->$k = $v;
				}
			}
		}
		public function jsonSerialize(){
			$boards = array();
			foreach($this->boards as $board){
				array_push($boards,array(
					'id'=>intval($board->id),
					'name'=>utf8_encode($board->name)
				));
			}
			return array(
				'id'=>intval($this->id),
				'name'=>utf8_encode($this->name),
				'boards'=>$boards
			);
		}
		public function __toString(){
			return "[Category {$this->id}] {\n  name: ".strip_tags($this->name)."\n}";
		}
		/**
		* Returns all the child boards of this category
		* 
		* @property boards
		* @type {Array}
		* @public
		*/
		public function __get($name){
			global $db_prefix;
			switch($name){
				case 'boards':
					$boards = $this->smf->sql->query("
						SELECT	id_board as id
						FROM {$db_prefix}boards b
						WHERE b.id_cat = ?
						ORDER BY board_order ASC;
					",'i',$this->id)->assoc_results;
					$a = array();
					foreach($boards as $board){
						array_push($a,$this->smf->board($board['id']));
					}
					return $a;
				break;
			}
		}
	}
?>