<?php
	require_once('sql.class.php');
	class Member implements JsonSerializable {
		public $smf;
		public $id;
		public $name;
		public $date_registered;
		public $post_count;
		public $id_group;
		public $real_name;
		public $signature;
		public $member_ip;
		public $member_ip2;
		public $id_post_group;
		public $additional_groups;
		public $usertitle;
		public $email_address;
		public $avatar;
		public function __construct($id,$smf){
			global $db_prefix;
			$this->smf = $smf;
			$this->id = $id;
			$member = $smf->sql->query("
				SELECT	member_name as name,
						date_registered,
						posts as post_count,
						id_group,
						real_name,
						signature,
						member_ip,
						member_ip2,
						id_post_group,
						additional_groups,
						usertitle,
						email_address,
						avatar
				FROM {$db_prefix}members m
				WHERE m.id_member = ?;
			",'i',$id)->assoc_result;
			if($member){
				foreach($member as $k => $v){
					$this->$k = $v;
				}
			}
		}
		public function jsonSerialize(){
			$groups = array();
			foreach($this->groups as $group){
				array_push($groups,array(
					'id'=>intval($group->id),
					'name'=>utf8_encode($group->name)
				));
			}
			return array(
				'id'=>intval($this->id),
				'name'=>utf8_encode($this->name),
				'real_name'=>$this->real_name,
				'date_registered'=>$this->date_registered,
				'post_count'=>$this->post_count,
				'colour'=>$this->colour,
				'signature'=>$this->signature,
				'member_ip'=>$this->member_ip,
				'member_ip2'=>$this->member_ip2,
				'usertitle'=>utf8_encode($this->usertitle),
				'email'=>utf8_encode($this->email),
				'group'=>array(
					'id'=>intval($this->group->id),
					'name'=>utf8_encode($this->group->name)
				),
				'groups'=>$groups,
				'avatar'=>$this->avatar_href
			);
		}
		public function __toString(){
			return "[Member {$this->id}] {\n  Name: {$this->name}\n  IP: {$this->member_ip}, {$this->member_ip2}\n  Posts: {$this->post_count}\n}";
		}
		public function __get($name){
			global $db_prefix;
			switch($name){
				case 'last_post':
					$post = $this->smf->sql->query("
						SELECT id_msg
						FROM {$db_prefix}messages
						WHERE id_member = ?
						ORDER BY poster_time DESC
						LIMIT 1;
					",'i',$this->id)->assoc_result;
					if($post){
						return $this->smf->post($post['id_msg']);
					}
				break;
				case 'first_post':
					$post = $this->smf->sql->query("
						SELECT id_msg
						FROM {$db_prefix}messages
						WHERE id_member = ?
						ORDER BY poster_time ASC
						LIMIT 1;
					",'i',$this->id)->assoc_result;
					if($post){
						return $this->smf->post($post['id_msg']);
					}
				break;
				case 'posts':
					$posts = $this->smf->sql->query("
						SELECT	id_msg as id
						FROM {$db_prefix}messages m
						WHERE m.id_member = ?;
					",'i',$this->id)->assoc_results;
					$a = array();
					foreach($posts as $post){
						array_push($a,$this->smf->post($post['id']));
					}
					return $a;
				break;
				case 'group':
					return $this->smf->group($this->id_group);
				break;
				case 'group_ids':
					return array_unique(
						array_merge(
							array(
								$this->id_group,
								$this->id_post_group
							),
							explode(',',$this->additional_groups)
						)
					);
				break;
				case 'groups':
					$gs = array();
					foreach($this->group_ids as $i){
						array_push($gs,$this->smf->group($i));
					}
					return $gs;
				break;
				case 'colour':case 'color':
					return $this->group->colour;
				break;
				case 'signature_html':
					return $this->smf->parse_bbc($this->signature,true);
				break;
				case 'api_key':
					$ourl = new Ourl();
					return $ourl->api_key($this->id);
				break;
				case 'email':
					return $this->email_address;
				break;
				case 'admin':
					return $this->id_group == 1 || in_array(1, $this->group_ids);
				break;
				case 'permissions':
					$ret = array();
					$rem = array();
					$groups = implode(',',$this->group_ids);
					$permissions = $this->smf->sql->query("
						SELECT permission, add_deny
						FROM {$db_prefix}permissions
						WHERE id_group IN (
							{$groups}
						);
					")->assoc_results;
					foreach($permissions as $permission){
						if(empty($permission['add_deny'])){
							array_push($rem,$permission['permission']);
						}else{
							array_push($ret,$permission['permission']);
						}
					}
					global $modSettings;
					if(!empty($modSettings['permission_enable_deny'])){
						$ret = array_diff($ret, $rem);
					}
					return $ret;
				break;
				case 'avatar_href':
					global $modSettings;
					if(empty($this->avatar)){
						$attach = $this->smf->sql->query("
							SELECT	id_attach,
									filename,
									attachment_type
							FROM {$db_prefix}attachments
							WHERE id_member = ?
						",'i',$this->id)->assoc_result;
						if($attach){
							if(empty($attach['attachment_type'])){
								global $scripturl;
								return $scripturl.'?action=dlattach;attach='.$attach['id_attach'].';type=avatar';
							}else{
								return $modSettings['custom_avatar_url'].'/'.$attach['filename'];
							}
						}else{
							return '';
						}
					}elseif(stristr($this->avatar,'http://') || stristr($this->avatar,'https://')){
						return $this->avatar;
					}else{
						return $modSettings['avatar_url'].'/'.$this->avatar;
					}
				break;
			}
		}
		public function pm($subject,$body,$member=null,$parent=0){
			$member = is_null($member)?$this->smf->current_user:($member instanceof Member?$member:$this->smf->member($member));
			return $this->smf->pm($this,null,$subject,$body,$member,$parent);
		}
		public function email($subject,$body,$html=false,$priority=3){
			return $this->smf->email($this,$subject,$body,$this->current_user,$html,$priority);
		}
		public function permission($permission){
			return $this->admin or (
				!is_array($permission) &&
				in_array($permission, $this->permissions)
			) or (
				is_array($permission) &&
				count(array_intersect($permission, $this->permissions)) != 0
			);
		}
	}
?>