<?php
	require_once('sql.class.php');
	class PrivateMessage implements JsonSerializable{
		public $smf;
		public $id_parent;
		public $id_member;
		public $from_name;
		public $subject;
		public $body;
		public function __construct($id,$smf){
			global $db_prefix;
			$this->smf = $smf;
			$this->id = $id;
			$pm = $smf->sql->query("
				SELECT	id_pm_head as id_parent,
						id_member_from as id_member,
						from_name,
						subject,
						body
				FROM {$db_prefix}personal_messages
				WHERE id_pm = ?;
			",'i',$id)->assoc_result;
			if($pm){
				foreach($pm as $k => $v){
					$this->$k = $v;
				}
			}
		}
		public function jsonSerialize(){
			$recipients = $this->recipients;
			$to = array();
			$bcc = array();
			foreach($recipients['to'] as $member){
				array_push($to,array(
					'id'=>intval($member->id),
					'real_name'=>utf8_encode($member->real_name),
					'name'=>utf8_encode($member->name)
				));
			}
			foreach($recipients['bcc'] as $member){
				array_push($bcc,array(
					'id'=>intval($member->id),
					'real_name'=>utf8_encode($member->real_name),
					'name'=>utf8_encode($member->name)
				));
			}
			return array(
				'id'=>intval($this->id),
				'subject'=>utf8_encode($this->subject),
				'from'=>array(
					'id'=>$this->member->id,
					'real_name'=>utf8_encode($this->member->real_name),
					'name'=>utf8_encode($this->member->name)
				),
				'to'=>$to,
				'bcc'=>$bcc
			);
		}
		public function __toString(){
			return "[PrivateMessage {$this->id}] {\n  Subject: {$this->subject}\n  By: {$this->member->name}\n}";
		}
		public function __get($name){
			global $db_prefix;
			switch($name){
				case 'member':
					return $this->smf->member($this->id_member);
				break;
				case 'parent':
					return $this->smf->pm($this->id_parent);
				break;
				case 'html':
					return $this->smf->parse_bbc($this->body);
				break;
				case 'recipient_ids':
					$a = $this->smf->sql->query("
						SELECT id_member as id,
							   bcc
						FROM {$db_prefix}pm_recipients
						WHERE id_pm = ?
						GROUP BY id_member, bcc;
					",'i',$this->id)->assoc_results;
					$r = array(
						'to'=>array(),
						'bcc'=>array()
					);
					foreach($a as $m){
						if($m['bcc']===0){
							array_push($r['to'],intval($m['id']));
						}else{
							array_push($r['bcc'],intval($m['id']));
						}
					}
					return $r;
				break;
				case 'recipients':
					$r = $this->recipient_ids;
					foreach($r['to'] as $k => $id){
						$r['to'][$k] = $this->smf->member($id);
					}
					foreach($r['bcc'] as $k => $id){
						$r['bcc'][$k] = $this->smf->member($id);
					}
					return $r;
				break;
				case 'permission':
					return $this->permission();
				break;
				case 'visible':
					return $this->visible();
				break;
			}
		}
		public function reply($body){
			return $this->smf->pm($this->member,null,$this->subject,$body,null,$this->parent);
		}
		public function replyall($body){
			$to = array($this->member);
			$bcc = array();
			foreach($this->recipients['to'] as $k => $member){
				if($member != $this->smf->current_user && $member != $this->member){
					array_push($to,$member);
				}
			}
			foreach($this->recipients['bcc'] as $k => $member){
				if($member != $this->smf->current_user && $member != $this->member){
					array_push($bcc,$member);
				}
			}
			return $this->smf->pm($to,$bcc,$this->subject,$body,null,$this->parent);
		}
		public function permission($member=null){
			if(is_null($member)){
				$member = $this->smf->current_user->id;
			}elseif($member instanceof Member){
				$member = $member->id;
			}
			$a = $this->smf->sql->query("
				SELECT id_member as id,
					   bcc
				FROM {$db_prefix}pm_recipients
				WHERE id_pm = ?
				GROUP BY id_member, bcc;
			",'i',$this->id)->assoc_results;
			array_push($a,intval($this->id_member));
			return $member->permission('pm_read') && in_array($member, $a);
		}
		public function visible($member=null){
			$member = is_null($member)?$this->smf->current_user:($member instanceof Member?$member:$this->smf->member($member));
			return $member->id == $this->member->id || in_array($member->id,$this->recipient_ids);
		}
	}
?>