<?php
	require_once('sql.class.php');
	class Post implements JsonSerializable{
		public $smf;
		public $id;
		public $id_topic;
		public $id_board;
		public $poster_time;
		public $id_member;
		public $id_msg_modified;
		public $subject;
		public $poster_name;
		public $poster_email;
		public $poster_ip;
		public $modified_time;
		public $modified_name;
		public $body;
		public $smileys;
		public $approved;
		public function __construct($id,$smf){
			global $db_prefix;
			$this->smf = $smf;
			$this->id = $id;
			$post = $smf->sql->query("
				SELECT	m.id_topic,
						m.id_board,
						m.poster_time,
						m.id_member,
						m.id_msg_modified,
						m.subject,
						IFNULL(u.real_name,m.poster_name) poster_name,
						IFNULL(u.email_address,m.poster_email) poster_email,
						IFNULL(u.member_ip,m.poster_ip) poster_ip,
						m.modified_time,
						m.modified_name,
						m.body,
						m.smileys_enabled as smileys,
						m.approved
				FROM {$db_prefix}messages m
				LEFT OUTER JOIN {$db_prefix}members u
					ON u.id_member = m.id_member
				WHERE m.id_msg = ?;
			",'i',$id)->assoc_result;
			if($post){
				foreach($post as $k => $v){
					$this->$k = $v;
				}
			}
		}
		public function jsonSerialize(){
			return array(
				'id'=>intval($this->id),
				'date_created'=>$this->poster_time,
				'date_modified'=>$this->modified_time,
				'subject'=>utf8_encode($this->subject),
				'body'=>utf8_encode($this->body),
				'smileys'=>intval($this->smileys)===1,
				'topic'=>array(
					'id'=>intval($this->topic->id),
					'subject'=>utf8_encode($this->topic->subject)
				),
				'member'=>array(
					'id'=>intval($this->member->id),
					'name'=>utf8_encode($this->member->name),
					'real_name'=>utf8_encode($this->member->real_name)
				),
				'approved'=>intval($this->approved)==1
			);
		}
		public function __toString(){
			return "[Post {$this->id}] {\n  Subject: {$this->subject}\n  By: {$this->member->name}\n}";
		}
		public function __get($name){
			global $db_prefix;
			switch($name){
				case 'member':
					return $this->smf->member($this->id_member);
				break;
				case 'topic':
					return $this->smf->topic($this->id_topic);
				break;
				case 'board':
					return $this->smf->board($this->id_board);
				break;
				case 'category':
					return $this->board->category;
				break;
				case 'html':
					return $this->smf->parse_bbc($this->body,$this->smileys==1);
				break;
				case 'permissions':
					return $this->permissions();
				break;
				case 'visible':
					return $this->visible();
				break;
				case 'attachment_ids':
					return $this->smf->sql->query("
						SELECT	a.id_attach as id
						FROM {$db_prefix}attachments a
						WHERE a.id_msg = ?
						AND attachment_type != 3;
					",'i',$this->id)->assoc_results;
				break;
				case 'attachments':
					$a = array();
					foreach($this->attachment_ids as $attachment){
						array_push($a,$this->smf->attachment($attachment['id']));
					}
					return $a;
				break;
				case 'thumbnail_ids':
					return $this->smf->sql->query("
						SELECT	a.id_attach as id
						FROM {$db_prefix}attachments a
						WHERE a.id_msg = ?
						AND attachment_type = 3;
					",'i',$this->id)->assoc_results;
				break;
				case 'thumbnails':
					$a = array();
					foreach($this->thumbnail_ids as $attachment){
						array_push($a,$this->smf->attachment($attachment['id']));
					}
					return $a;
				break;
			}
		}
		public function read($member=null){
			$member = is_null($member)?$this->smf->current_user:$member;
			global $db_prefix;
			$this->smf->sql->query("
				INSERT INTO {$db_prefix}log_topics (
					id_msg,
					id_member,
					id_topic
				)
				VALUES (?,?,?)
				ON DUPLICATE KEY UPDATE
					id_msg = IF(VALUES(id_msg)>id_msg,VALUES(id_msg),id_msg);
			",'iii',
				$this->id,
				$member->id,
				$this->topic->id
			)->execute();
		}
		public function permission($permission,$member=null){
			return $this->topic->permission($permission,$member);
		}
		public function permissions($member=null){
			return $this->topic->permissions($member);
		}
		public function visible($member=null){
			return $this->topic->visible($member);
		}
	}
?>