<?php
	chdir(realpath(dirname(__FILE__)));
	if(!defined('SMF_PATH')){
		define('SMF_PATH','../');
	}
	require_once(SMF_PATH.'/SSI.php');
	require_once(SMF_PATH.'/Sources/Subs-Post.php');
	require_once('sql.class.php');
	require_once('category.class.php');
	require_once('board.class.php');
	require_once('topic.class.php');
	require_once('post.class.php');
	require_once('attachment.class.php');
	require_once('pm.class.php');
	require_once('member.class.php');
	require_once('group.class.php');
	require_once('page.class.php');
	/**
	* SMF class. Allows you to interface with the forum's database.
	*
	* @module smf
	* @class SMF
	* @constructor
	*/
	class SMF implements JsonSerializable {
		/**
		* The SQL object for the SMF database
		*
		* @property sql
		* @type {SQL}
		* @public
		* @required
		*/
		public $sql;
		/**
		* An array of the boards on the site
		*
		* @property boards
		* @type {Array}
		* @public
		* @default Array()
		* @required
		*/
		private $boards = array();
		private $categories = array();
		private $topics = array();
		private $posts = array();
		private $attachments = array();
		private $pms = array();
		private $members = array();
		private $groups = array();
		private $pages = array();
		public function __construct(){
			global $db_server, $db_user, $db_passwd, $db_name;
			$this->sql = new SQL($db_server, $db_user, $db_passwd, $db_name);
		}
		public function __get($name){
			global $db_prefix, $context;
			switch($name){
				case 'boards':
					return $this->boards;
				break;
				/**
				* An array of the categories on the site
				*
				* @property categories
				* @type {Array}
				* @public
				* @default Array()
				* @required
				*/
				case 'categories':
					$a = $this->sql->query("
						SELECT	id_cat AS id
						FROM {$db_prefix}categories
						ORDER BY cat_order ASC;
					")->assoc_results;
					foreach($a as $k => $v){
						$a[$k] = $v['id'];
					}
					$a = array_unique($a);
					$r = array();
					foreach($a as $id){
						array_push($r,$this->category($id));
					}
					return $r;
				break;
				case 'groups':
					$a = $this->sql->query("
						SELECT	id_group AS id
						FROM {$db_prefix}membergroups;
					")->assoc_results;
					foreach($a as $k => $v){
						$a[$k] = $v['id'];
					}
					$a = array_unique($a);
					$r = array();
					foreach($a as $id){
						array_push($r,$this->group($id));
					}
					return $r;
				break;
				case 'bots':
					$a = $this->sql->query("
						SELECT id
						FROM {$db_prefix}bots;
					")->assoc_results;
					foreach($a as $k => $v){
						$a[$k] = $v['id'];
					}
					$a = array_unique($a);
					$r = array();
					foreach($a as $id){
						array_push($r,$this->bot($id));
					}
					return $r;
				break;
				case 'networks':
					$a = $this->sql->query("
						SELECT id
						FROM {$db_prefix}networks;
					")->assoc_results;
					foreach($a as $k => $v){
						$a[$k] = $v['id'];
					}
					$a = array_unique($a);
					$r = array();
					foreach($a as $id){
						array_push($r,$this->network($id));
					}
					return $r;
				break;
				case 'pages':
					$a = $this->sql->query("
						SELECT id_page AS id
						FROM {$db_prefix}dp_dream_pages;
					")->assoc_results;
					foreach($a as $k => $v){
						$a[$k] = $v['id'];
					}
					$a = array_unique($a);
					$r = array();
					foreach($a as $id){
						array_push($r,$this->page($id));
					}
					return $r;
				break;
				case 'name':
					return $context['forum_name'];
				break;
				case 'error':
					return $this->sql->error;
				break;
				case 'current_user':
					return $this->member($context['user']['id']);
				break;
				case 'post_count':
					$r = $this->sql->query("
						SELECT COUNT(id_msg) AS postcount
						FROM {$db_prefix}messages
					")->assoc_result;
					return $r['postcount'];
				break;
			}
		}
		public function jsonSerialize(){
			return array(
				'name'=>$this->name
			);
		}
		public function __toString(){
			return "[SMF] {\n  Name: {$this->name}\n}";
		}
		public function __call($name, $args){
			// Support deprecated get_<name> calls
			if(substr($name,0,4) == 'get_'){
				return call_user_func_array(array($this,substr($name,4)),$args);
			}
		}
		public function empty_cache(){
			$this->boards = array();
			$this->categories = array();
			$this->topics = array();
			$this->posts = array();
			$this->attachments = array();
			$this->pms = array();
			$this->members = array();
			$this->groups = array();
			$this->bots = array();
			$this->networks = array();
			$this->pages = array();
			return $this;
		}
		/**
		* Returns a category for the specified ID
		*
		* @method category
		* @param {Integer} id The category ID
		* @return {Category} Returns the category specified by the ID. Otherwise false.
		*/
		public function category($id){
			if(!isset($this->categories[$id])){
				$this->categories[$id] = new Category($id,$this);
			}
			return $this->categories[$id];
		}
		/**
		* Returns a board for the specified ID
		*
		* @method board
		* @param {Integer} id The board ID.
		* @return {Board} Returns the board for the specified ID. Otherwise false.
		*/
		public function board($id){
			if(!isset($this->boards[$id])){
				$this->boards[$id] = new Board($id,$this);
			}
			return $this->boards[$id];
		}
		public function topic($id){
			global $db_prefix;
			if(!is_numeric($id)){
				$a = $this->sql->query("
					SELECT id_topic as id
					FROM {$db_prefix}pretty_topic_urls
					WHERE pretty_url = ?;
				",'s',$id)->assoc_result;
				if($a){
					$id = intval($a['id']);
				}
			}
			if(!isset($this->topics[$id])){
				$this->topics[$id] = new Topic($id,$this);
			}
			return $this->topics[$id];
		}
		public function attachment($id){
			if(!isset($this->attachments[$id])){
				$this->attachments[$id] = new Attachment($id,$this);
			}
			return $this->attachments[$id];
		}
		public function member($id){
			if(!isset($this->members[$id])){
				$this->members[$id] = new Member($id,$this);
			}
			return $this->members[$id];
		}
		public function group($id){
			if(!isset($this->groups[$id])){
				$this->groups[$id] = new Group($id,$this);
			}
			return $this->groups[$id];
		}
		public function page($id){
			if(!isset($this->pages[$id])){
				$this->pages[$id] = new Page($id,$this);
			}
			return $this->pages[$id];
		}
		public function header($title=null){
			if(!is_null($title)){
				$this->title($title);
			}
			template_html_above();
			template_body_above();
			return $this;
		}
		public function title($title){
			global $context;
			$context['page_title_html_safe'] = $title;
			return $this;
		}
		public function footer(){
			template_body_below();
			template_html_below();
			return $this;
		}
		public function parse_bbc($string,$smileys=true){
			return parse_bbc($string,$smileys);
		}
		public function parse_smileys($string){
			parseSmileys($string);
			return $string;
		}
		public function add_link($name,$url){
			global $context;
			array_push($context['linktree'],array(
				'url'=>$url,
				'name'=>$name
			));
			return $this;
		}
		public function post(){
			if(func_num_args()==1){
				$id = func_get_arg(0);
				if(!isset($this->posts[$id])){
					$this->posts[$id] = new Post($id,$this);
				}
				return $this->posts[$id];
			}else{
				$subject = func_get_arg(0);
				$body = func_get_arg(1);
				$topic = func_get_arg(2);
				if($topic === false){
					$topic = null;
				}
				$board = func_get_arg(3);
				if(func_num_args() != 5 || func_get_arg(4)){
					$user = $this->current_user;
				}
				$id = func_num_args() == 6 ? func_get_arg(5) : false;
				if($id === false){
					$id = null;
				}
				$mo = array(
					'id' => is_null($id)?0:$id,
					'subject' => $subject,
					'body' => $body,
					'icon' => '',
					'smileys_enabled' => true,
					'sig_enabled' => 1,
					'attachments' => array(),
					'approved' => true,
				);
				$to = array(
					'id' => is_null($topic)?0:$topic->id,
					'board' => $board->id,
					'poll' => is_null($topic)?null:$topic->id_poll,
					'lock_mode' => is_null($topic)?null:$topic->locked,
					'sticky_mode' => is_null($topic)?null:$topic->is_sticky,
					'mark_as_read' => true,
					'is_approved' => true,
				);
				$po = array(
					'id' => $user->id,
					'name' => $user->name,
					'email' => $user->email,
					'update_post_count' => true,
				);
				if(is_null($id)){
					createPost($mo,$to,$po);
				}else{
					$mo['modify_time'] = time();
					$mo['modify_name'] = $user->name;
					modifyPost($mo,$to,$po);
				}
				return $this->post($mo['id']);
			}
		}
		public function pm(){
			if(func_num_args()==1){
				$id = func_get_arg(0);
				if(!isset($this->pms[$id])){
					$this->pms[$id] = new PrivateMessage($id,$this);
				}
				return $this->pms[$id];
			}else{
				$to = func_get_arg(0);
				$bcc = func_get_arg(1);
				$subject = func_get_arg(2);
				$body = func_get_arg(3);
				$from = func_num_args() > 4?func_get_arg(4):null;
				$parent = func_num_args() > 5?func_get_arg(5):false;
				$recipients = array(
					'to'=>array(),
					'bcc'=>array()
				);
				if($to instanceof Member){
					array_push($recipients['to'],intval($to->id));
				}elseif(is_array($to) && $to[0] instanceof Member){
					foreach($to as $k => $member){
						array_push($recipients['to'],intval($member->id));
					}
				}elseif(!is_null($to)){
					$recipients['to'] = intval($to);
				}
				if($bcc === false){
					$bcc = null;
				}elseif($bcc instanceof Member){
					array_push($recipients['bcc'],intval($bcc->id));
				}elseif(is_array($bcc) && $bcc[0] instanceof Member){
					foreach($bcc as $k => $member){
						array_push($recipients['bcc'],intval($member->id));
					}
				}elseif(!is_null($bcc)){
					$recipients['bcc'] = intval($bcc);
				}
				if($parent instanceof PrivateMessage){
					$parent = intval($parent->id);
				}elseif($parent === false){
					$parent = 0;
				}
				if(is_int($from)){
					$from = $this->member($from);
				}elseif($from === false){
					$from = $this->current_user;
				}
				if($from instanceof Member){
					$from = array(
						'id'=>intval($from->id),
						'name'=>$from->real_name,
						'username'=>$from->name
					);
				}
				return sendpm($recipients, $subject, $body,false, $from, $parent);
			}
		}
		public function email($to,$subject,$body,$from=null,$html=false,$priority=3){
			$recipients = array();
			if($to instanceof Member){
				array_push($recipients,$to->email);
			}elseif(is_array($to) && $to[0] instanceof Member){
				foreach($to as $k=>$member){
					array_push($recipients,$member->email);
				}
			}else{
				$recipients = $to;
			}
			return sendmail($recipients,$subject,$body,null,$from,$html,$priority);
		}
	}
?>
