<?php
	require_once('query.class.php');
	/**
	* SQL class. Used for handling SQL connections
	*
	* @module sql
	* @class SQL
	* @constructor
	*/
	class SQL {
		/**
		* This is the mysqli connection beneath everything
		*
		* @property sql
		* @type {mysqli}
		* @private
		* @required
		*/
		private $sql;
		public function __construct($server,$user,$pass,$db){
			$this->sql = new mysqli($server,$user,$pass,$db) or die('Unable to connect to mysql');
		}
		public function __invoke(){
			return $this->sql;
		}
		public function __get($name){
			switch($name){
				case 'error':
					return $this->sql->error;
				break;
			}
		}
		/**
		* Returns a Query object based on inputs
		*
		* @method query
		* @param {String} sql The sql expression to run
		* @param {String=null} [types] A string containing all the types of arguments being passed
		* @param {Mixed} [bindings]* The bindings to use in the sql statement
		* @return {Query} Returns the query object
		*/
		public function query(){
			$reflect = new ReflectionClass('Query');
			$args = array_merge(array($this),func_get_args());
			return $reflect->newInstanceArgs($args);
		}
		public function escape($s){
			return $this->sql->escape_string($s);
		}
		public function charset($charset){
			return $this->sql->set_charset($charset);
		}
	}
?>
